#!/usr/bin/env python
# -*- coding: utf-8 -*-


### # -*- coding: iso-8859-1 -*-

# Movie Jukebox by AliAbdul
from Components.ActionMap import ActionMap, NumberActionMap
from Components.config import (
    config,
    ConfigLocations,
    ConfigSelection,
    ConfigSubsection,
    ConfigText, 
    ConfigYesNo,
)
from Components.Label import Label
from Components.MenuList import MenuList
from Components.MultiContent import (
    MultiContentEntryText,
    MultiContentEntryPixmapAlphaBlend,
    MultiContentEntryPixmapAlphaTest,
)
from Components.Pixmap import Pixmap
from Components.PluginComponent import plugins
from enigma import (
    eListboxPythonMultiContent,
    ePicLoad,
    eServiceCenter,
    eServiceReference,
    eTimer,
    getDesktop,
    gFont,
    RT_HALIGN_CENTER,
    RT_VALIGN_CENTER,
    RT_WRAP,
)
from os import (
    environ,
    listdir,
    path as os_path,
    remove,
    stat,
    system
)

from Plugins.Plugin import PluginDescriptor
from Screens.ChoiceBox import ChoiceBox
from Screens.InfoBar import MoviePlayer
from Screens.LocationBox import (LocationBox, MovieLocationBox)
from Screens.MessageBox import MessageBox
from Screens.Screen import Screen
from Tools.BoundFunction import boundFunction
from Tools.Directories import (
    fileExists,
    pathExists,
    resolveFilename,
    SCOPE_HDD,
    SCOPE_LANGUAGE,
    SCOPE_PLUGINS,
    SCOPE_SKIN_IMAGE,
)
from Tools.LoadPixmap import LoadPixmap
from Tools.NumericalTextInput import NumericalTextInput
import gettext
import os
# import stat
import time
# from builtins import chr
try:
    unichr
except NameError:
    unichr = chr


try:
    from Plugins.Extensions.IMDb.plugin import IMDB
except ImportError:
    pass


class TagEditorList(MenuList):
    def __init__(self):
        MenuList.__init__(self, [], False, eListboxPythonMultiContent)
        self.l.setItemHeight(25)
        self.l.setFont(0, gFont("Regular", 20))
        self.onSelectionChanged = []
        self.list = []

    def changeCheckState(self):
        idx = self.l.getCurrentSelectionIndex()
        count = 0
        newList = []
        for x in self.list:
            if idx == count:
                if x[1] is True:
                    newList.append([x[0], False])
                else:
                    newList.append([x[0], True])
            else:
                newList.append([x[0], x[1]])
            count += 1
        del self.list
        self.list = newList
        self.SetList(self.list)

    def entry(self, name, selected):
        res = [(name, selected)]
        if selected:
            icon = LoadPixmap(cached=True, path=resolveFilename(SCOPE_SKIN_IMAGE, "skin_default/icons/lock_on.png"))
        else:
            icon = LoadPixmap(cached=True, path=resolveFilename(SCOPE_SKIN_IMAGE, "skin_default/icons/lock_off.png"))
        res.append(MultiContentEntryPixmapAlphaTest(pos=(0, 0), size=(25, 25), png=icon))
        res.append(MultiContentEntryText(pos=(30, 3), size=(270, 25), font=0, text=name))
        return res

    def SetList(self, list):
        self.list = list
        newList = []
        for x in self.list:
            newList.append(self.entry(x[0], x[1]))
        self.l.setList(newList)


class TagEditor(Screen):
    skin = """
            <screen position="center,center" size="360,360" title="%s">
                <widget name="list" position="5,5" size="350,350" scrollbarMode="showOnDemand" />
            </screen>
            """ % _("Tag Editor")

    def __init__(self, session, moviescannerTags, movieTags):
        Screen.__init__(self, session)

        if moviescannerTags:
            self.moviescannerTags = moviescannerTags
        else:
            self.moviescannerTags = []
        if movieTags:
            self.movieTags = movieTags
        else:
            self.movieTags = []
        self["list"] = TagEditorList()

        self["actions"] = ActionMap(["MovieJukeboxActions"],
                                    {"back": self.close,
                                     "ok": self.ok,
                                     "left": self["list"].changeCheckState,
                                     "right": self["list"].changeCheckState}, -1)

        self.onLayoutFinish.append(self.SetList)

    def ok(self):
        list = []
        for x in self["list"].list:
            if x[1] is True:
                list.append(x[0])
        self.close(list)

    def SetList(self):
        list = []
        for x in self.moviescannerTags:
            if x != ("* %s *" % _("All")):
                if x in self.movieTags:
                    list.append([x, True])
                else:
                    list.append([x, False])
        self["list"].SetList(list)


class DesktopSize:
    def __init__(self):
        desktop = getDesktop(0)
        size = desktop.size()
        self.width = size.width()
        self.height = size.height()


desktopsize = DesktopSize()


try:
    from LT.LTMisc import getCurrentSkinPath
    test = "%s/%s" % (resolveFilename(SCOPE_SKIN_IMAGE, getCurrentSkinPath()), "/mj/noCover.png")
    if fileExists(test):
        PNG_PATH = resolveFilename(SCOPE_SKIN_IMAGE, getCurrentSkinPath()) + "/mj/"
    else:
        PNG_PATH = resolveFilename(SCOPE_PLUGINS) + "/Extensions/MovieJukebox/"
except:
    PNG_PATH = resolveFilename(SCOPE_PLUGINS) + "/Extensions/MovieJukebox/"


config.plugins.MovieJukebox = ConfigSubsection()
config.plugins.MovieJukebox.background = ConfigText(default=resolveFilename(SCOPE_PLUGINS) + "/Extensions/MovieJukebox/default_%dx%d.png" % (desktopsize.width, desktopsize.height), fixed_size=False)
config.plugins.MovieJukebox.sort = ConfigSelection(choices=["name_asc", "name_desc", "date_asc", "date_desc"], default="name_asc")
config.plugins.MovieJukebox.bookmarks = ConfigSelection(choices=["all", "selected", "user"], default="all")
config.plugins.MovieJukebox.locations = ConfigLocations(default=["/media/hdd/movie/"])
config.plugins.MovieJukebox.show_extralabel = ConfigYesNo(default=True)
config.plugins.MovieJukebox.show_subdirs = ConfigYesNo(default=True)


class Movie:
    def __init__(self, path, filename, name, tags, info, isDvd=False, isDirectory=False):
        self.filename = "%s/%s" % (path, filename)
        self.filename = self.filename.replace("//", "/")
        self.name = name
        self.icon = "%s/%s.png" % (path, filename)
        self.icon = self.icon.replace("//", "/")
        self.tags = tags
        self.info = info
        self.isDvd = isDvd
        self.isDirectory = isDirectory
        if isDirectory:
            self.filesize = "N/A"
            file_stats = None
        elif isDvd:
            self.filesize = "N/A"
            try:
                file_stats = os.stat("%s/VIDEO_TS/VIDEO_TS.IFO" % self.filename)
            except:
                file_stats = None
        else:
            file_stats = os.stat(self.filename)
            try:
                self.filesize = "%d MB" % ((int(file_stats[stat.ST_SIZE]) / 1024) / 1024)
            except:
                file_stats = None
        if file_stats:
            # self.lastmodified = time.strftime("%d.%m.%Y %I:%M %p", time.localtime(file_stats[stat.ST_MTIME]))
            self.lastaccessed = time.strftime("%d.%m.%Y %I:%M %p", time.localtime(file_stats[stat.ST_ATIME]))
            self.creationtime = time.strftime("%d.%m.%Y %I:%M %p", time.localtime(file_stats[stat.ST_CTIME]))
            self.date = file_stats[stat.ST_CTIME]
        else:
            # self.lastmodified = "N/A"
            self.lastaccessed = "N/A"
            self.creationtime = "N/A"
            self.date = "N/A"

    def getNameWithoutExtension(self):
        if self.isDvd or self.isDirectory or self.filename.endswith(".ts"):
            return self.name
        else:
            name = self.name
            if name.__contains__("."):
                while not name.endswith("."):
                    name = name[:-1]
                name = name[:-1]
            return name

    def changeTags(self, tags):
        self.tags = tags
        content = ""
        try:
            f = open(self.filename + ".imdb", "r")
            lines = f.read().split("\n")
            f.close()
        except:
            lines = []
        firstLine = True
        for line in lines:
            if firstLine:
                for tag in tags:
                    if tag != "" and tag != " " and tag != "\n" and tag != "|":
                        content = content + tag + " "
                content = content[:-1] + "\n"
                firstLine = False
            else:
                content = content + line + "\n"
        try:
            f = open(self.filename + ".imdb", "w")
            f.write(content)
            f.close()
        except:
            print("[Movie Jukebox] Error writing imdb file")


class TagList(MenuList):
    def __init__(self):
        MenuList.__init__(self, [], False, eListboxPythonMultiContent)
        self.l.setItemHeight(30)
        self.l.setFont(0, gFont("Regular", 25))
        self.updateSelectedEntryPixmapAllowed = True
        self.active = False
        self.tags = []

    def setActive(self, active):
        self.active = active

    def update(self):
        self.SetList(self.tags)

    def selectionChanged(self):
        if self.updateSelectedEntryPixmapAllowed:
            self.updateSelectedEntryPixmap()

    def updateSelectedEntryPixmap(self):
        idx = self.getSelectionIndex()
        list = []
        count = 0
        for x in self.list:
            if idx == count:
                selected = True
            else:
                selected = False
            list.append(self.entry(x[0], selected))
            count += 1
        self.updateSelectedEntryPixmapAllowed = False
        self.l.setList(list)
        self.updateSelectedEntryPixmapAllowed = True

    def entry(self, text, selected):
        res = [(text)]
        if self.active:
            pngSelected = PNG_PATH + "active_tag.png"
        else:
            pngSelected = PNG_PATH + "selected_tag.png"
        if selected:
            res.append(MultiContentEntryPixmapAlphaTest(pos=(0, 0), size=(200, 30), png=LoadPixmap(cached=True, path=pngSelected)))
        res.append(MultiContentEntryText(pos=(10, 2), size=(180, 26), font=0, text=text, flags=RT_HALIGN_CENTER))
        return res

    def SetList(self, l):
        del self.tags
        self.tags = l
        list = []
        selected = True
        for x in l:
            list.append(self.entry(x, selected))
            selected = False
        self.setList(list)

    def getCurrent(self):
        return self.l.getCurrentSelection()[0]

    def select(self, index):
        if len(self.list) > 0:
            self.moveToIndex(index)


class AlphabetList(MenuList):
    def __init__(self):
        MenuList.__init__(self, [], False, eListboxPythonMultiContent)
        self.l.setItemHeight(45)
        self.l.setFont(0, gFont("Regular", 25))
        self.index = 0
        self.letters = []
        self.maxWidth = None
        self.active = False

    def setActive(self, active):
        self.active = active

    def update(self):
        self.SetList(self.letters)

    def entry(self, list, selected):
        res = [("")]
        if self.active:
            pngSelected = PNG_PATH + "active_alpha.png"
        else:
            pngSelected = PNG_PATH + "selected_alpha.png"
        calculate = False
        if self.maxWidth:
            width = self.maxWidth
            widthCalculate = width
        else:
            width = desktopsize.width
            widthCalculate = width - 100
        if (len(list) * 45) > (widthCalculate):
            calculate = True
            maxEntries = (widthCalculate) / 45
            if ((selected * 45) + 50) < (width / 2):
                firstEntry = 0
            elif (((len(self.letters) - selected) * 45) - 50) < (width / 2):
                firstEntry = len(self.letters) - maxEntries
            else:
                firstEntry = selected - (maxEntries / 2)
        idx = 0
        idxCalculate = 0
        for letter in list:
            if calculate:
                if idx >= firstEntry:
                    if selected == idx:
                        res.append(MultiContentEntryPixmapAlphaTest(pos=(idxCalculate * 45, 0), size=(45, 45), png=LoadPixmap(cached=True, path=pngSelected)))
                    res.append(MultiContentEntryText(pos=(idxCalculate * 45, 0), size=(45, 45), font=0, text=letter, flags=RT_HALIGN_CENTER | RT_VALIGN_CENTER))
                    idxCalculate += 1
                idx += 1
            else:
                if selected == idx:
                    res.append(MultiContentEntryPixmapAlphaTest(pos=(idx * 45, 0), size=(45, 45), png=LoadPixmap(cached=True, path=pngSelected)))
                res.append(MultiContentEntryText(pos=(idx * 45, 0), size=(45, 45), font=0, text=letter, flags=RT_HALIGN_CENTER | RT_VALIGN_CENTER))
                idx += 1
        return res

    def SetList(self, list):
        del self.letters
        self.letters = list
        if self.index > len(list):
            self.index = len(list) - 1
        self.setList([self.entry(list, self.index)])

    def next(self):
        self.index += 1
        if self.index > (len(self.letters) - 1):
            self.index = 0
        self.SetList(self.letters)

    def previous(self):
        self.index -= 1
        if self.index < 0:
            self.index = (len(self.letters) - 1)
        self.SetList(self.letters)

    def getCurrentLetter(self):
        return self.letters[self.index]

    def getSelectedIndex(self):
        return self.index

    def select(self, index):
        self.index = index
        self.setList([self.entry(self.letters, index)])

    def moveToLetter(self, letter):
        sel = -1
        idx = 0
        for x in self.letters:
            if x == letter:
                sel = idx
                break
            idx += 1
        if sel > -1:
            self.select(sel)
        else:
            self.select(self.index)

    def applySkin(self, desktop, screen):
        if self.skinAttributes is not None:
            for (attrib, value) in self.skinAttributes:
                if attrib == "size":
                    self.maxWidth = int(value.split(",")[0])
                    break
        return MenuList.applySkin(self, desktop, screen)


class MovieList(MenuList):
    def __init__(self):
        MenuList.__init__(self, [], False, eListboxPythonMultiContent)
        self.l.setItemHeight(160)
        self.l.setFont(0, gFont("Regular", 18))
        self.selectedEntry = 0
        self.maxHorizontalEntries = int((desktopsize.width - 300) / 210)
        maxVerticalEntries = int((desktopsize.height - 200) / 160)
        self.maxEntries = self.maxHorizontalEntries * maxVerticalEntries
        self.movies = []
        self.onSelectionChanged = None
        self.active = False

    def setActive(self, active):
        self.active = active

    def update(self):
        self.SetList(self.movies)

    def entry(self, movies):
        res = [(movies)]
        hEntry = 0
        for movie in movies:
            if fileExists(movie[0].icon):
                res.append(MultiContentEntryPixmapAlphaBlend(pos=(hEntry * 210, 0), size=(210, 160), png=LoadPixmap(movie[0].icon)))
                showText = False
            else:
                if movie[0].isDirectory:
                    noCoverName = "noCoverDirectory.png"
                else:
                    noCoverName = "noCover.png"
                res.append(MultiContentEntryPixmapAlphaBlend(pos=(hEntry * 210, 0), size=(210, 160), png=LoadPixmap(cached=True, path=PNG_PATH + noCoverName)))
                showText = True
            if movie[1] is True:
                if self.active:
                    pngNameSelected = "active_movie.png"
                else:
                    pngNameSelected = "selected_movie.png"
                res.append(MultiContentEntryPixmapAlphaBlend(pos=(hEntry * 210, 0), size=(210, 160), png=LoadPixmap(cached=True, path=PNG_PATH + pngNameSelected)))
            if showText:
                res.append(MultiContentEntryText(pos=((hEntry * 210) + 20, 20), size=(170, 120), font=0, text=movie[0].name, flags=RT_HALIGN_CENTER | RT_VALIGN_CENTER | RT_WRAP))
            hEntry += 1
        return res

    def SetList(self, movies, selected=None):
        if selected is not None:
            self.selectedEntry = selected
        del self.movies
        self.movies = movies
        entryStart = 0
        while True:
            if (self.selectedEntry >= entryStart) and (self.selectedEntry < (entryStart + self.maxEntries)):
                break
            else:
                entryStart += self.maxEntries
        idx = 0
        added = 0
        maxEntriesReached = False
        list = []
        for movie in movies:
            if idx >= entryStart:
                if idx == self.selectedEntry:
                    list.append([movie, True])
                else:
                    list.append([movie, False])
                added += 1
                if added > self.maxEntries:
                    maxEntriesReached = True
                    break
            idx += 1
        movieList = []
        tmpList = []
        added = 0
        for movie in list:
            if added >= self.maxHorizontalEntries:
                movieList.append(self.entry(tmpList))
                del tmpList
                tmpList = []
                added = 0
                tmpList.append(movie)
            else:
                tmpList.append(movie)
            added += 1
        if (len(tmpList) > 0) and (maxEntriesReached is False):
            movieList.append(self.entry(tmpList))
        self.setList(movieList)
        self.selectionChanged()

    def first(self):
        if self.selectedEntry > 0:
            self.selectedEntry = 0
            self.SetList(self.movies)
            self.selectionChanged()

    def previous(self):
        if self.selectedEntry > 0:
            self.selectedEntry -= 1
            self.SetList(self.movies)
            self.selectionChanged()

    def next(self):
        if self.selectedEntry < (len(self.movies) - 1):
            self.selectedEntry += 1
            self.SetList(self.movies)
            self.selectionChanged()

    def previousBigStep(self):
        if (self.selectedEntry - self.maxHorizontalEntries) >= 0:
            self.selectedEntry -= self.maxHorizontalEntries
            self.SetList(self.movies)
            self.selectionChanged()

    def nextBigStep(self):
        if (self.selectedEntry + self.maxHorizontalEntries) < len(self.movies):
            self.selectedEntry += self.maxHorizontalEntries
            self.SetList(self.movies)
            self.selectionChanged()

    def getCurrentMovie(self):
        if (len(self.movies) > 0) and (self.selectedEntry < len(self.movies)):
            return self.movies[self.selectedEntry]
        elif (len(self.movies) > 0) and (self.selectedEntry >= len(self.movies)):
            self.selectedEntry = len(self.movies) - 1
            return self.movies[self.selectedEntry]
        else:
            return None

    def selectionChanged(self):
        if self.onSelectionChanged:
            self.onSelectionChanged()


class MetaFileHandler:
    def __init__(self, dir, file):
        self.name = None
        content = ""
        meta = "%s/%s.meta" % (dir, file)
        meta = meta.replace("//", "/")
        if fileExists(meta):
            try:
                f = open(meta, "r")
                content = f.read()
                f.close()
            except:
                pass
        lines = content.split("\n")
        if len(lines) > 1:
            self.name = lines[1]


class ImdbMetaFileHandler:
    def __init__(self, dir, file):
        self.tags = None
        self.info = ""
        meta = "%s/%s.imdb" % (dir, file)
        meta = meta.replace("//", "/")
        if fileExists(meta):
            tag = True
            try:
                f = open(meta, "r")
                for l in f:
                    if tag:
                        tag = False
                        self.tags = []
                        tmp = l.split(" ")
                        for x in tmp:
                            if x != "" and x != " " and x != "\n" and x != "|":
                                self.tags.append(x.replace("\n", ""))
                    else:
                        self.info += l.replace(" | ", " ")
                f.close()
            except:
                pass


class MovieScanner:
    def __init__(self):
        self.letters = []
        self.tags = []
        self.movies = []
        self.currDir = ""
        self.baseDir = ""
        self.lastDir = ""
        self.playable = ["avi", "dat", "divx", "iso", "m2ts", "m4a", "mkv", "mp4", "mov", "mpg", "ts", "vob"]
        self.digits = ["0", "1", "2", "3", "4", "5", "6", "7", "8", "9"]
        self.allowedLetters = []
        allowedLetters = ["a", "b", "c", "d", "e", "f", "g", "h", "i", "j", "k", "l", "m", "n", "o", "p", "q", "r", "s", "t", "u", "v", "w", "x", "y", "z"]
        for x in allowedLetters:
            self.allowedLetters.append(x)
            self.allowedLetters.append(x.upper())

    def scanMovies(self, callback):
        del self.letters
        self.letters = ["*"]
        del self.tags
        self.tags = ["* %s *" % _("All")]
        del self.movies
        self.movies = []
        if self.currDir != "":
            if self.currDir == self.baseDir:
                self.currDir = ""
                self.baseDir = ""
        if self.currDir == "":
            if config.plugins.MovieJukebox.bookmarks.value == "all":
                bookmarks = config.movielist.videodirs
                dirs = bookmarks and bookmarks.value[:] or []
            elif config.plugins.MovieJukebox.bookmarks.value == "selected":
                dirs = [config.movielist.last_videodir.value]
            else:
                locations = config.plugins.MovieJukebox.locations
                dirs = locations and locations.value[:] or []
        else:
            dirs = [self.currDir]
            self.movies.append(Movie("", "", "..", None, "", False, True))
        for dir in dirs:
            try:
                files = listdir(dir)
            except:
                files = []

            for file in files:
                subdir = ("%s/%s" % (dir, file)).replace("//", "/")
                if (not os_path.isdir(subdir)) and file.__contains__("."):  # Movie
                    ext = file.split(".")[-1]
                    if ext.lower() in self.playable:
                        name = file
                        if ext == "ts":
                            metafilehandler = MetaFileHandler(dir, file)
                            if metafilehandler.name:
                                letter = metafilehandler.name[:1]
                                name = metafilehandler.name
                            else:
                                letter = file[:1]
                        else:
                            letter = file[:1]
                        if letter not in self.allowedLetters:
                            try:
                                test = u'%s' % letter
                            except:
                                letter = name[:2]
                        if letter not in self.letters:
                            if letter in self.digits:
                                if "09" not in self.letters:
                                    self.letters.append("09")
                            else:
                                letter = letter.upper()
                                self.letters.append(letter)
                        imdbmetafilehandler = ImdbMetaFileHandler(dir, file)
                        tags = imdbmetafilehandler.tags
                        info = imdbmetafilehandler.info
                        if tags:
                            for tag in tags:
                                if (tag not in self.tags) and tag != "|":
                                    self.tags.append(tag)
                        self.movies.append(Movie(dir, file, name, tags, info))
                elif os_path.isdir(subdir):  # Directory
                    if pathExists("%s/VIDEO_TS" % subdir) and fileExists("%s/VIDEO_TS/VIDEO_TS.IFO" % subdir):  # DVD Directory
                        name = (subdir).split("/")[-1]
                        letter = name[:1]
                        if letter not in self.allowedLetters:
                            try:
                                test = u'%s' % name
                            except:
                                letter = (subdir).split("/")[-1][:2]
                        if letter not in self.letters:
                            if letter in self.digits:
                                if "09" not in self.letters:
                                    self.letters.append("09")
                            else:
                                letter = letter.upper()
                                self.letters.append(letter)
                        imdbmetafilehandler = ImdbMetaFileHandler(dir, file)
                        tags = imdbmetafilehandler.tags
                        info = imdbmetafilehandler.info
                        if tags:
                            for tag in tags:
                                if (tag not in self.tags) and tag != "|":
                                    self.tags.append(tag)
                        self.movies.append(Movie(dir, file, name, tags, info, True))
                    elif config.plugins.MovieJukebox.show_subdirs.value:  # Normal directory
                        name = (subdir).split("/")[-1]
                        if not name.startswith(".") and name != "lost+found" and name != "usr":
                            letter = name[:1]
                            if letter not in self.allowedLetters:
                                try:
                                    test = u'%s' % name
                                except:
                                    letter = (subdir).split("/")[-1][:2]
                            if letter not in self.letters:
                                if letter in self.digits:
                                    if "09" not in self.letters:
                                        self.letters.append("09")
                                else:
                                    letter = letter.upper()
                                    self.letters.append(letter)
                            imdbmetafilehandler = ImdbMetaFileHandler(dir, file)
                            tags = imdbmetafilehandler.tags
                            info = imdbmetafilehandler.info
                            if tags:
                                for tag in tags:
                                    if (tag not in self.tags) and tag != "|":
                                        self.tags.append(tag)
                            self.movies.append(Movie(dir, file, name, tags, info, False, True))

        self.sortList(self.movies)
        self.letters.sort()
        self.tags.sort()
        callback()

    def getMovies(self, letter, tag):
        list = []
        for movie in self.movies:
            name = movie.name
            movieLetter = name[:1]
            if movieLetter not in self.allowedLetters:
                try:
                    test = u'%s' % name[:1]
                except:
                    movieLetter = name[:2]
            if letter == "*" or ((movieLetter in self.digits) and letter == "09") or (movieLetter.upper() == letter):
                if (tag == ("* %s *" % _("All"))) or (movie.tags and (tag in movie.tags)):
                    list.append(movie)
        self.sortList(list)
        return list

    def sortList(self, movies):
        if config.plugins.MovieJukebox.sort.value == "name_asc":
            return movies.sort(key=self.buildSortNameKey, reverse=False)
        elif config.plugins.MovieJukebox.sort.value == "name_desc":
            return movies.sort(key=self.buildSortNameKey, reverse=True)
        elif config.plugins.MovieJukebox.sort.value == "date_asc":
            return movies.sort(key=self.buildSortDateKey, reverse=False)
        elif config.plugins.MovieJukebox.sort.value == "date_desc":
            return movies.sort(key=self.buildSortDateKey, reverse=True)

    def buildSortNameKey(self, x):
        try:
            name = x.name
        except:
            name = ""
        return (name and name.lower() or "")

    def buildSortDateKey(self, x):
        try:
            name = str(x.date)
        except:
            name = ""
        return (name and name.lower() or "")

    def topDir(self):
        if "/" in self.currDir:
            self.lastDir = self.currDir.split("/")[-1]
            while not self.currDir.endswith("/"):
                self.currDir = self.currDir[:-1]
            self.currDir = self.currDir[:-1]

    def saveBaseDir(self):
        if self.baseDir == "":
            self.baseDir = self.currDir
            if "/" in self.baseDir:
                while not self.baseDir.endswith("/"):
                    self.baseDir = self.baseDir[:-1]
                self.baseDir = self.baseDir[:-1]

    def getLastDirIndex(self):
        idx = 0
        cur = 0
        for movie in self.movies:
            name = movie.name
            if name == self.lastDir:
                cur = idx
                break
            else:
                idx += 1
        return cur

    def createPlaylist(self, dir, files):
        l = []
        for x in files:
            l.append(("%s/%s" % (dir, x)).replace("//", "/"))
        l.sort()
        return l

    def getPlaylist(self, dir):
        list = []
        try:
            files = listdir(dir)
        except:
            files = []
        for file in files:
            ext = file.split(".")[-1]
            if (ext.lower() in self.playable) and (not ext.lower().endswith(".iso")):
                list.append(file)
        if (len(list) < 2):
            return self.createPlaylist(dir, list)
        else:
            if fileExists(dir + ".serie"):
                return self.createPlaylist(dir, list)
            else:
                cdParts = True
                for entry in list:
                    if not entry.lower().startswith("cd"):
                        cdParts = False
                if cdParts:
                    return self.createPlaylist(dir, list)
        return []


moviescanner = MovieScanner()


class ChangedMoviePlayer(MoviePlayer):
    def __init__(self, session, playlist):
        self.session = session
        self.playlist = playlist
        if isinstance(playlist, list):
            self.isPlaylist = True
            self.current = 0
            first = playlist[0]
            if first.endswith(".ts"):
                type = 1
            else:
                type = 4097
            ref = eServiceReference(type, 0, first)
            MoviePlayer.__init__(self, session, ref)
        else:
            self.isPlaylist = False
            MoviePlayer.__init__(self, session, playlist)
        self.skinName = "MoviePlayer"
        self["extended_actions"] = ActionMap(["MovieJukeboxActions"],
                                             {"channelUp": self.nextPlaylistEntry,
                                              "channelDown": self.previousPlaylistEntry}, -1)

    def leavePlayer(self):
        self.session.openWithCallback(self.leavePlayerConfirmed, MessageBox, _("Stop playing this movie?"))

    def leavePlayerConfirmed(self, answer):
        if answer:
            self.close()

    def doEofInternal(self, playing):
        if self.execing and playing:
            self.nextPlaylistEntry()

    def getPluginList(self):
        list = []
        for p in plugins.getPlugins(where=PluginDescriptor.WHERE_EXTENSIONSMENU):
            if p.name != _("Movie Jukebox"):
                list.append(((boundFunction(self.getPluginName, p.name), boundFunction(self.runPlugin, p), lambda: True), None))
        return list

    def showMovies(self):
        if self.isPlaylist and len(self.playlist) > 1:
            list = []
            idx = 0
            for entry in self.playlist:
                name = entry.split("/")[-1]
                list.append((name, idx))
                idx += 1
            self.session.openWithCallback(self.playlistEntryCallback, ChoiceBox, title=_("Select..."), list=list)

    def playlistEntryCallback(self, callback):
        if callback is not None:
            self.current = callback[1]
            self.playPlaylistEntry()

    def playPlaylistEntry(self):
        selected = self.playlist[self.current]
        if selected.endswith(".ts"):
            type = 1
        else:
            type = 4097
        ref = eServiceReference(type, 0, selected)
        self.session.nav.playService(ref)

    def previousPlaylistEntry(self):
        if self.isPlaylist and len(self.playlist) > 1:
            if self.current > 0:
                self.current -= 1
                self.playPlaylistEntry()

    def nextPlaylistEntry(self):
        if self.isPlaylist and len(self.playlist) > 1:
            if (self.current + 1) < len(self.playlist):
                self.current += 1
                self.playPlaylistEntry()


class NumberTextScreen(Screen):
    skin = """
        <screen position="75,75" size="100,25" flags="wfNoBorder" >
            <widget name="text" position="0,0" size="100,25"  noWrap="1" valign="center" halign="center" />
        </screen>"""

    def __init__(self, session):
        Screen.__init__(self, session)
        self["text"] = Label("")
        self.hideTimer = eTimer()
        if os.path.exists('/var/lib/dpkg/info'):
            self.hideTimer_conn = self.hideTimer.timeout.connect(self.doHide)
        else:
            self.hideTimer.callback.append(self.doHide)
        # self.hideTimer.callback.append(self.doHide)
        self.callback = None
        self.letter = ""

    def showLabel(self, letter):
        self.hideTimer.stop()
        self.letter = letter
        self["text"].setText(letter)
        self.show()
        self.hideTimer.start(3000, 1)

    def doHide(self):
        self.hide()
        if self.callback:
            self.callback(self.letter)
        self["text"].setText("")
        self.letter = ""


class MovieJukeboxLocationBox(LocationBox):
    def __init__(self, session):
        LocationBox.__init__(self, session, _("Please select the locations..."), "", "/media", config.plugins.MovieJukebox.locations)
        self.skinName = "LocationBox"


class MovieJukebox(Screen):
    ALPHA = 2
    MOVIE = 0
    TAG = 1

    def __init__(self, session):
        self.session = session

        if desktopsize.height > 576:  # Some skin corrections to not be totally unusable with SD-skins
            tagListHeight = 450
            extraMovielistPlace = 60
        else:
            tagListHeight = 300
            extraMovielistPlace = 0
        self.skin = """
            <screen position="0,0" size="%d,%d" flags="wfNoBorder" >
                <widget name="background" position="0,0" size="%d,%d" zPosition="-1" alphatest="on" />
                <widget name="alphabetList" position="50,50" size="%d,45" transparent="1" selectionDisabled="1" />
                <widget name="tagList" position="50,110" size="200,%d" transparent="1" selectionDisabled="1" />
                <widget name="movieList" position="%d,110" size="%d,%d" transparent="1" selectionDisabled="1" />
                <widget name="fakeList" position="0,0" size="0,0" />
                <ePixmap pixmap="%s" position="50,%d" size="10,15" alphatest="on" />
                <ePixmap pixmap="%s" position="240,%d" size="10,15" alphatest="on" />
                <widget name="curList" position="65,%d" size="170,25" transparent="1" font="Regular;22" noWrap="1" valign="center" halign="center" />
                <widget name="extraLabel" position="50,%d" size="%d,25" transparent="1" font="Regular;22" noWrap="1" />
            </screen>""" % (desktopsize.width, desktopsize.height,
                            desktopsize.width, desktopsize.height,
                            desktopsize.width - 100,
                            tagListHeight,
                            270 + extraMovielistPlace, desktopsize.width - 300, desktopsize.height - 200,
                            resolveFilename(SCOPE_PLUGINS) + "/Extensions/MovieJukebox/arrow_r.png", desktopsize.height - 95,
                            resolveFilename(SCOPE_PLUGINS) + "/Extensions/MovieJukebox/arrow_r.png", desktopsize.height - 95,
                            desktopsize.height - 100,
                            desktopsize.height - 70, desktopsize.width - 100)
        Screen.__init__(self, session)

        self["alphabetList"] = AlphabetList()
        self["background"] = Pixmap()
        self["curList"] = Label()
        self["extraLabel"] = Label()
        self["fakeList"] = MenuList([])
        self["movieList"] = MovieList()
        self["movieList"].onSelectionChanged = self.buildExtraLabel
        self["tagList"] = TagList()
        self.extraLabelTimer = eTimer()
        if os.path.exists('/var/lib/dpkg/info'):
            self.extraLabelTimer_conn = self.extraLabelTimer.timeout.connect(self.updateExtraLabel)
        else:
            self.extraLabelTimer.callback.append(self.updateExtraLabel)
        # self.extraLabelTimer.callback.append(self.updateExtraLabel)
        self.extraLabel = ""
        self.currentExtraLabel = ""

        self.numberTextDialog = self.session.instantiateDialog(NumberTextScreen)
        self.numberTextDialog.hide()
        self.numberTextDialog.callback = self.letterSelected

        self["actions"] = NumberActionMap(["MovieJukeboxActions"],
                                          {"back": self.exit,
                                           "ok": self.ok,
                                           "left": self.left,
                                           "right": self.right,
                                           "up": self.up,
                                           "down": self.down,
                                           "previousList": self.previousList,
                                           "nextList": self.nextList,
                                           "doPathSelect": self.doPathSelect,
                                           "searchImdb": self.searchImdb,
                                           "showMenu": self.showMenu,
                                           "channelUp": self.previousTag,
                                           "channelDown": self.nextTag,
                                           "gotAsciiCode": self.keyAsciiCode,
                                           "0": self.firstEntry,
                                           "1": self.keyNumberGlobal,
                                           "2": self.keyNumberGlobal,
                                           "3": self.keyNumberGlobal,
                                           "4": self.keyNumberGlobal,
                                           "5": self.keyNumberGlobal,
                                           "6": self.keyNumberGlobal,
                                           "7": self.keyNumberGlobal,
                                           "8": self.keyNumberGlobal,
                                           "9": self.keyNumberGlobal}, -1)

        self.working = False
        self.firstStart = True
        self.currentList = self.MOVIE
        self.lastservice = session.nav.getCurrentlyPlayingServiceReference()
        self.oldTagSelected = 0
        self.oldAlphaSelected = 0

        self.numericalTextInput = NumericalTextInput()
        self.numericalTextInput.setUseableChars(u'1234567890ABCDEFGHIJKLMNOPQRSTUVWXYZ')

        if not pathExists(config.movielist.last_videodir.value):
            config.movielist.last_videodir.value = resolveFilename(SCOPE_HDD)
            config.movielist.last_videodir.save()

        moviescanner.currDir = ""
        self.onLayoutFinish.append(self.updateMovies)
        self.onShow.append(self._onShow)
        self.onHide.append(self._onHide)

    def _onShow(self):
        if config.plugins.MovieJukebox.show_extralabel.value:
            self.extraLabelTimer.start(500, 1)

    def _onHide(self):
        if config.plugins.MovieJukebox.show_extralabel.value:
            self.extraLabelTimer.stop()

    def buildExtraLabel(self):
        self.extraLabel = ""
        self.currentExtraLabel = ""
        movie = self["movieList"].getCurrentMovie()
        if movie:
            try:
                if config.plugins.MovieJukebox.show_extralabel.value:
                    tmp = movie.info.split("\n")
                    tmp.append("%s: %s" % (_("Filesize"), movie.filesize))
                    # tmp.append("%s: %s" % (_("Last modified"), movie.lastmodified))
                    tmp.append("%s: %s" % (_("Last accessed"), movie.lastaccessed))
                    tmp.append("%s: %s" % (_("Creation time"), movie.creationtime))
                    tmp.insert(0, movie.name)
                    self.extraLabel = ("   ") * 20
                    for x in tmp:
                        if x != "" and x != " " and x != "\n":
                            self.extraLabel = self.extraLabel + " *** " + x
                    self.extraLabel = self.extraLabel + " *** "
                    self.currentExtraLabel = self.extraLabel
                else:
                    self["extraLabel"].setText(movie.name)
            except Exception as e:
                print('errorr access:', e)

    def updateExtraLabel(self):
        if config.plugins.MovieJukebox.show_extralabel.value:
            if len(self.currentExtraLabel) > 2:
                self.currentExtraLabel = self.currentExtraLabel[1:]
            if (len(self.currentExtraLabel) < (len(self.extraLabel) * 2)) and (self.extraLabel != ""):
                self.currentExtraLabel += self.extraLabel
            self["extraLabel"].setText(self.currentExtraLabel)
            self.extraLabelTimer.start(500, 1)
        else:
            self.extraLabelTimer.stop()

    def switchExtraLabelValue(self):
        if config.plugins.MovieJukebox.show_extralabel.value:
            config.plugins.MovieJukebox.show_extralabel.value = False
        else:
            config.plugins.MovieJukebox.show_extralabel.value = True
        config.plugins.MovieJukebox.show_extralabel.save()
        self.buildExtraLabel()
        self.updateExtraLabel()
        self.working = False

    def updateMovies(self):
        self.working = True
        self.oldTagSelected = 0
        self.oldAlphaSelected = 0
        self["tagList"].select(0)
        self["alphabetList"].index = 0
        moviescanner.scanMovies(self.updateMoviesCallback)

    def updateMoviesCallback(self):
        self["tagList"].SetList(moviescanner.tags)
        self["alphabetList"].SetList(moviescanner.letters)
        self["movieList"].SetList(moviescanner.movies)
        self.currentList = self.MOVIE
        self["tagList"].selectionEnabled(0)
        self["alphabetList"].selectionEnabled(0)
        self["movieList"].selectionEnabled(0)
        self["fakeList"].selectionEnabled(1)
        self.working = False

        if self.firstStart:
            self.firstStart = False
            if fileExists(config.plugins.MovieJukebox.background.value):
                background = config.plugins.MovieJukebox.background.value
            else:
                background = resolveFilename(SCOPE_PLUGINS) + "/Extensions/MovieJukebox/default_%dx%d.png" % (desktopsize.width, desktopsize.height)
            ptr = LoadPixmap(background)
            if ptr:
                self["background"].instance.setPixmap(ptr)
            else:
                self.picload = ePicLoad()
                self.picload.PictureData.get().append(self.setBackgroundImage)
                self.picload.setPara((desktopsize.width, desktopsize.height, 1, 1, False, 1, "#00000000"))
                self.picload.startDecode(background)
            self.updateCurrentListName()

    def setBackgroundImage(self, picInfo=None):
        if picInfo:
            ptr = self.picload.getData()
            if ptr is not None:
                self["background"].instance.setPixmap(ptr)

    def exit(self):
        if self.working is False:
            self.numberTextDialog.callback = None
            self.numberTextDialog.hide()
            self.session.deleteDialog(self.numberTextDialog)
            self.close()

    def ok(self, movie=None):
        if self.working is False:
            if movie is None:
                movie = self["movieList"].getCurrentMovie()
            if self.currentList == self.MOVIE:
                if movie:
                    if movie.isDirectory:
                        playPlaylist = False
                        if movie.name == "..":
                            moviescanner.topDir()
                        else:
                            playlist = moviescanner.getPlaylist(movie.filename)
                            if len(playlist):
                                playPlaylist = True
                                self.session.open(ChangedMoviePlayer, playlist)
                            else:
                                moviescanner.currDir = movie.filename
                                moviescanner.saveBaseDir()
                        if not playPlaylist:
                            self.updateMovies()
                            if moviescanner.lastDir != "":
                                self["movieList"].selectedEntry = moviescanner.getLastDirIndex()
                            else:
                                self["movieList"].selectedEntry = 0
                            self.updateMovieList()
                    elif movie.isDvd or movie.filename.endswith(".iso") or movie.filename.endswith(".ISO"):
                        if fileExists(resolveFilename(SCOPE_PLUGINS) + "/Extensions/DVDPlayer/plugin.py"):
                            from Plugins.Extensions.DVDPlayer.plugin import DVDPlayer
                            self.session.openWithCallback(self.moviePlayerCallback, DVDPlayer, None, [movie.filename])
                        else:
                            self.session.open(MessageBox, _("DVDPlayer is not installed!"), MessageBox.TYPE_INFO)
                    else:
                        if movie.filename.endswith(".ts") or movie.filename.endswith(".TS"):
                            type = 1
                        else:
                            type = 4097
                        ref = eServiceReference(type, 0, movie.filename)
                        self.session.openWithCallback(self.moviePlayerCallback, ChangedMoviePlayer, ref)
            else:
                if self.currentList == self.TAG:
                    self.oldTagSelected = self["tagList"].getSelectedIndex()
                else:
                    self.oldAlphaSelected = self["alphabetList"].getSelectedIndex()
                movies = moviescanner.getMovies(self["alphabetList"].getCurrentLetter(), self["tagList"].getCurrent())
                if movie:
                    idx = 0
                    cur = 0
                    for x in movies:
                        if x.filename == movie.filename:
                            cur = idx
                            break
                        idx += 1
                else:
                    cur = 0
                self["movieList"].SetList(movies, cur)
                self.currentList = self.MOVIE
                self.updateCurrentListName()

    def moviePlayerCallback(self, callback=None):
        self.session.nav.playService(self.lastservice)

    def left(self):
        if self.working is False:
            if self.currentList == self.MOVIE:
                self["movieList"].previous()
            elif self.currentList == self.TAG:
                self["tagList"].pageUp()
            elif self.currentList == self.ALPHA:
                self["alphabetList"].previous()

    def right(self):
        if self.working is False:
            if self.currentList == self.MOVIE:
                self["movieList"].next()
            elif self.currentList == self.TAG:
                self["tagList"].pageDown()
            elif self.currentList == self.ALPHA:
                self["alphabetList"].next()

    def up(self):
        if self.working is False:
            if self.currentList == self.MOVIE:
                self["movieList"].previousBigStep()
            elif self.currentList == self.TAG:
                self["tagList"].up()

    def down(self):
        if self.working is False:
            if self.currentList == self.MOVIE:
                self["movieList"].nextBigStep()
            elif self.currentList == self.TAG:
                self["tagList"].down()

    def previousList(self):
        if self.working is False:
            self.handleOldSelectedEntries()
            self.currentList -= 1
            if self.currentList < 0:
                self.currentList = self.ALPHA
            self.updateCurrentListName()

    def nextList(self):
        if self.working is False:
            self.handleOldSelectedEntries()
            self.currentList += 1
            if self.currentList > self.ALPHA:
                self.currentList = self.MOVIE
            self.updateCurrentListName()

    def firstEntry(self, number):
        if self.working is False:
            self["movieList"].first()

    def keyNumberGlobal(self, number):
        if self.working is False:
            if self.currentList == self.MOVIE:
                unichar = self.numericalTextInput.getKey(number)
                charstr = unichar.encode("utf-8")
                if len(charstr) == 1:
                    self.moveToChar(charstr[0])

    def keyAsciiCode(self):
        if self.working is False:
            if self.currentList == self.MOVIE:
                # from builtins import chr
                unichar = unichr(getPrevAsciiCode())
                charstr = unichar.encode("utf-8")
                if len(charstr) == 1:
                    self.moveToChar(charstr[0])

    def moveToChar(self, char):
        self.numberTextDialog.showLabel(char)

    def letterSelected(self, letter):
        if (letter in moviescanner.letters) or (letter in moviescanner.digits):
            self.working = True
            if letter in moviescanner.digits:
                letter = "09"
            self["alphabetList"].moveToLetter(letter)
            self.currentList = self.ALPHA
            self.working = False
            self.ok()

    def updateActivePictures(self):
        if self.currentList == self.MOVIE:
            self["movieList"].setActive(True)
            self["tagList"].setActive(False)
            self["alphabetList"].setActive(False)
        elif self.currentList == self.TAG:
            self["movieList"].setActive(False)
            self["tagList"].setActive(True)
            self["alphabetList"].setActive(False)
        else:
            self["movieList"].setActive(False)
            self["tagList"].setActive(False)
            self["alphabetList"].setActive(True)
        self["movieList"].update()
        self["tagList"].update()
        self["alphabetList"].update()

    def updateCurrentListName(self):
        if self.currentList == self.MOVIE:
            self["curList"].setText(_("Movies"))
        elif self.currentList == self.TAG:
            self["curList"].setText(_("Tags"))
        else:
            self["curList"].setText(_("Alphabet"))
        self.updateActivePictures()

    def handleOldSelectedEntries(self):
        nowTagSelected = self["tagList"].getSelectedIndex()
        if nowTagSelected != self.oldTagSelected:
            self["tagList"].select(self.oldTagSelected)
        nowAlphaSelected = self["alphabetList"].getSelectedIndex()
        if nowAlphaSelected != self.oldAlphaSelected:
            self["alphabetList"].select(self.oldAlphaSelected)

    def deleteMovie(self):
        movie = self["movieList"].getCurrentMovie()
        if movie is not None:
            self.session.openWithCallback(self.deleteConfirmed, MessageBox, (_("Delete %s?") % movie.name))

    def deleteConfirmed(self, callback):
        if callback:
            movie = self["movieList"].getCurrentMovie()
            if movie.isDvd or movie.isDirectory:
                try:
                    system('rm -R "%s"' % movie.filename)
                except:
                    pass
            else:
                filename = movie.filename
                ext = filename.split(".")[-1]
                if ext == "ts":
                    ref = eServiceReference(1, 0, filename)
                    offline = eServiceCenter.getInstance().offlineOperations(ref)
                    if offline is not None:
                        if offline.deleteFromDisk(0):
                            pass
                else:
                    try:
                        remove(filename)
                    except:
                        pass
            for ext in [".imdb", ".png"]:
                try:
                    remove(movie.filename + ext)
                except:
                    pass
            self["movieList"].selectedEntry = 0
            self.updateMovies()
        else:
            self.working = False

    def doPathSelect(self):
        self.session.openWithCallback(self.doPathSelectCallback, MovieLocationBox, _("Please select the movie path..."), config.movielist.last_videodir.value)

    def doPathSelectCallback(self, callback):
        if callback and (callback != config.movielist.last_videodir.value):
            config.movielist.last_videodir.value = callback
            config.movielist.last_videodir.save()
            self["movieList"].selectedEntry = 0
            self.updateMovies()
        else:
            self.working = False

    def searchImdb(self, asked=False):
        movie = self["movieList"].getCurrentMovie()
        if movie:
            if movie.name != "..":
                if movie.info == "":
                    self.session.openWithCallback(self.searchImdbCallback, IMDB, movie.getNameWithoutExtension(), callbackNeeded=True)
                elif (movie.info != "") and (asked is True):
                    self.session.openWithCallback(self.searchImdbCallback, IMDB, movie.getNameWithoutExtension(), callbackNeeded=True)
                else:
                    self.session.openWithCallback(self.searchImdbQuestionCallback, MessageBox, _("There are already IMDb informations. Search again?"))
            else:
                self.working = False
        else:
            self.working = False

    def searchImdbQuestionCallback(self, callback):
        if callback:
            self.searchImdb(True)
        else:
            self.working = False

    def searchImdbCallback(self, data=None):
        if data:
            self.imdb_data = data
            self.session.openWithCallback(self.searchImdbCallback2, MessageBox, _("Did you find the movie you searched for?"))
        else:
            self.working = False

    def searchImdbCallback2(self, yesno):
        if yesno:
            text = "%s\n%s" % (self.imdb_data[1], self.imdb_data[0])
            if text != "\n":
                self.working = True
                movie = self["movieList"].getCurrentMovie()
                try:
                    f = open(movie.filename + ".imdb", "w")
                    f.write(text)
                    f.close()
                except:
                    pass
                tmp = (self.imdb_data[1]).split(" ")
                del movie.tags
                movie.tags = []
                for x in tmp:
                    if x != "" and x != " " and x != "\n" and x != "|":
                        movie.tags.append(x)
                self.updateMovieList()
        else:
            self.working = False

    def backgroundSwitcher(self):
        from Plugins.Extensions.BackgroundSwitcher.plugin import BackgroundSwitcher
        self.session.openWithCallback(self.backgroundSwitcherCallback, BackgroundSwitcher)

    def backgroundSwitcherCallback(self, callback=None):
        if callback and callback != config.plugins.MovieJukebox.background.value:
            config.plugins.MovieJukebox.background.value = callback
            config.plugins.MovieJukebox.background.save()
            self["background"].instance.setPixmapFromFile(callback)
        self.working = False

    def coverManager(self):
        movie = self["movieList"].getCurrentMovie()
        if movie:
            from Plugins.Extensions.CoverManager.plugin import CoverManager
            self.session.openWithCallback(self.coverManagerCallback, CoverManager, movie.getNameWithoutExtension())
        else:
            self.working = False

    def coverManagerCallback(self, callback=None):
        if callback:
            movie = self["movieList"].getCurrentMovie()
            if movie:
                system('mv -f /tmp/cover.png "%s.png"' % (movie.filename))
                movie.icon = "%s.png" % movie.filename
                self.updateMovieList()
        else:
            self.working = False

    def burnDVD(self):
        movie = self["movieList"].getCurrentMovie()
        ref = eServiceReference(1, 0, movie.filename)
        from Plugins.Extensions.DVDBurn.plugin import main
        dvdburn = main(self.session)
        dvdburn.selectedSource(ref)
        self.working = False

    def cutlistEditor(self):
        movie = self["movieList"].getCurrentMovie()
        ref = eServiceReference(1, 0, movie.filename)
        from Plugins.Extensions.CutListEditor.plugin import CutListEditor
        self.session.openWithCallback(self.doNothing, CutListEditor, ref)

    def showMenu(self):
        if (self.currentList == self.MOVIE) and (self.working is False):
            self.working = True
            list = []
            movie = self["movieList"].getCurrentMovie()
            list.append((_("Quick list"), self.quickList))
            if movie:
                if movie.filename.endswith(".ts"):
                    if movie.isDirectory is False and movie.isDvd is False:
                        if fileExists(resolveFilename(SCOPE_PLUGINS) + "/Extensions/CutListEditor/plugin.py"):
                            list.append((_("Cutlist Editor"), self.cutlistEditor))
                        if fileExists(resolveFilename(SCOPE_PLUGINS) + "/Extensions/DVDBurn/plugin.py"):
                            list.append((_("Burn to DVD..."), self.burnDVD))
                        if fileExists("/usr/lib/enigma2/python/Screens/LTMovielistPlugins.pyc") or fileExists("/usr/lib/enigma2/python/Screens/LTMovielistPlugins.pyo"):
                            list.append((_("lock movie"), self.lockMovie))
                            list.append((_("unlock movie"), self.unlockMovie))
                if movie.name != "..":
                    list.append((_("Search IMDb"), self.searchImdb))
                    if fileExists(resolveFilename(SCOPE_PLUGINS) + "/Extensions/CoverManager/plugin.py"):
                        list.append((_("Cover Manager"), self.coverManager))
                    list.append((_("Tag Editor"), self.tagEditor))
                if movie.isDirectory and not movie.isDvd:
                    if fileExists(movie.filename + ".serie"):
                        list.append((_("Remove series marker"), self.removeSeriesMarker))
                    else:
                        list.append((_("Mark as series"), self.addSeriesMarker))
            list.append((_("Sort by name ascending"), self.sortByNameAscending))
            list.append((_("Sort by name descending"), self.sortByNameDescending))
            list.append((_("Sort by date ascending"), self.sortByDateAscending))
            list.append((_("Sort by date descending"), self.sortByDateDescending))
            list.append((_("Select movie path"), self.doPathSelect))
            if fileExists(resolveFilename(SCOPE_PLUGINS) + "/Extensions/BackgroundSwitcher/plugin.py"):
                list.append((_("Background Switcher"), self.backgroundSwitcher))
            list.append((_("Show movies from selected bookmark"), self.toggleBookmarkSelected))
            list.append((_("Show movies from all bookmarks"), self.toggleBookmarkAll))
            list.append((_("Show movies from user selected locations"), self.toggleBookmarkUser))
            list.append((_("Select user locations"), self.selectLocations))
            if config.plugins.MovieJukebox.show_extralabel.value:
                list.append((_("Don't show extended informations"), self.switchExtraLabelValue))
            else:
                list.append((_("Show extended informations"), self.switchExtraLabelValue))
            if moviescanner.currDir == "":
                if config.plugins.MovieJukebox.show_subdirs.value:
                    list.append((_("Don't show sub directories"), self.switchSubDirConfig))
                else:
                    list.append((_("Show sub directories"), self.switchSubDirConfig))
            if movie:
                if movie.name != "..":
                    list.append((_("Delete movie"), self.deleteMovie))
            list.append((_("close"), self.doNothing))
            self.session.openWithCallback(self.showMenuCallback, ChoiceBox, title=_("Select..."), list=list)

    def previousTag(self):
        if self.working is False:
            if self.currentList == self.MOVIE:
                self["tagList"].up()
                self.currentList = self.TAG
                self.ok()

    def nextTag(self):
        if self.working is False:
            if self.currentList == self.MOVIE:
                self["tagList"].down()
                self.currentList = self.TAG
                self.ok()

    def showMenuCallback(self, callback):
        if callback is not None:
            callback[1]()
        else:
            self.doNothing()

    def doNothing(self):
        self.working = False

    def sortByNameAscending(self):
        config.plugins.MovieJukebox.sort.value = "name_asc"
        config.plugins.MovieJukebox.sort.save()
        self.updateMovieList()

    def sortByNameDescending(self):
        config.plugins.MovieJukebox.sort.value = "name_desc"
        config.plugins.MovieJukebox.sort.save()
        self.updateMovieList()

    def sortByDateAscending(self):
        config.plugins.MovieJukebox.sort.value = "date_asc"
        config.plugins.MovieJukebox.sort.save()
        self.updateMovieList()

    def sortByDateDescending(self):
        config.plugins.MovieJukebox.sort.value = "date_desc"
        config.plugins.MovieJukebox.sort.save()
        self.updateMovieList()

    def updateMovieList(self):
        movie = self["movieList"].getCurrentMovie()
        movies = moviescanner.getMovies(self["alphabetList"].getCurrentLetter(), self["tagList"].getCurrent())
        if movie:
            idx = 0
            cur = 0
            for x in movies:
                if x.filename == movie.filename:
                    cur = idx
                    break
                idx += 1
        else:
            cur = 0
        self["movieList"].SetList(movies, cur)
        self.working = False

    def lockMovie(self):
        from Screens.LTMovielistPlugins import lockMovie
        movie = self["movieList"].getCurrentMovie()
        ref = eServiceReference(1, 0, movie.filename)
        lockMovie(self.session, ref)
        self.working = False

    def unlockMovie(self):
        from Screens.LTMovielistPlugins import unlockMovie
        movie = self["movieList"].getCurrentMovie()
        ref = eServiceReference(1, 0, movie.filename)
        unlockMovie(self.session, ref)
        self.working = False

    def removeSeriesMarker(self):
        movie = self["movieList"].getCurrentMovie()
        try:
            system('rm -R "%s.serie"' % movie.filename)
        except:
            pass
        self.working = False

    def addSeriesMarker(self):
        movie = self["movieList"].getCurrentMovie()
        try:
            system('touch "%s.serie"' % movie.filename)
        except:
            pass
        self.working = False

    def toggleBookmark(self, value):
        config.plugins.MovieJukebox.bookmarks.value = value
        config.plugins.MovieJukebox.bookmarks.save()
        self["movieList"].selectedEntry = 0
        self.updateMovies()
        self.updateMovieList()

    def toggleBookmarkSelected(self):
        self.toggleBookmark("selected")

    def toggleBookmarkAll(self):
        self.toggleBookmark("all")

    def toggleBookmarkUser(self, ret=None):
        self.toggleBookmark("user")

    def selectLocations(self):
        self.session.openWithCallback(self.toggleBookmarkUser, MovieJukeboxLocationBox)

    def quickList(self):
        list = []
        for movie in moviescanner.movies:
            if movie.isDirectory is False:
                list.append((movie.name, movie))
        if len(list) > 0:
            self.session.openWithCallback(self.quickListCallback, ChoiceBox, title=_("Select..."), list=list)
        else:
            self.working = False

    def quickListCallback(self, movie):
        self.working = False
        if movie:
            self.ok(movie[1])

    def tagEditor(self):
        movie = self["movieList"].getCurrentMovie()
        if movie:
            self.session.openWithCallback(self.tagEditorCallback, TagEditor, moviescanner.tags, movie.tags)

    def tagEditorCallback(self, movieTags=None):
        if movieTags:
            if len(movieTags) > 0:
                movie = self["movieList"].getCurrentMovie()
                del movie.tags
                movie.tags = []
                movie.changeTags(movieTags)
                self.updateMovies()
        self.working = False

    def switchSubDirConfig(self):
        if config.plugins.MovieJukebox.show_subdirs.value:
            config.plugins.MovieJukebox.show_subdirs.value = False
        else:
            config.plugins.MovieJukebox.show_subdirs.value = True
        config.plugins.MovieJukebox.show_subdirs.save()
        self["movieList"].selectedEntry = 0
        self.updateMovies()
        self.updateMovieList()


def start(session, **kwargs):
    session.open(MovieJukebox)


def Plugins(**kwargs):
    return PluginDescriptor(name=_("Movie Jukebox"), description=_("Movie Jukebox with 3D DVD cover"), where=[PluginDescriptor.WHERE_EXTENSIONSMENU, PluginDescriptor.WHERE_PLUGINMENU], fnc=start, icon="plugin.png")
